@@ -708,6 +708,34 @@ sub draw_form_field_text {
 
 ################################################################################
 
+=head1 draw_form_field_hidden
+
+    'text': C<textarea>.   - C<draw_form>.
+
+=head2 
+
+=over
+
+=item name
+
+ CGI-      C<$data>.
+
+=item value
+
+   ,   C<!$data{$$options{name}}>.
+
+=back
+
+=cut
+
+################################################################################
+
+sub draw_form_field_hidden {
+	drawer_call ('draw_form_field_hidden', @_);
+}
+
+################################################################################
+
 =head1 draw_form_field_checkbox
 
     C<checkbox>   1.   - C<draw_form>.
