@@ -1,5 +1,7 @@
 use DBI;
 
+use Data::Dumper;
+
 ################################################################################
 
 sub sql_do {
@@ -28,7 +30,7 @@ sub sql_select_all_cnt {
 	if ($sql =~ s{LIMIT.*}{}ism) {
 #		pop @params;
 	}
-	my $st = $db -> prepare ($sql);
+	$st = $db -> prepare ($sql);
 	$st -> execute (@params);
 	my $cnt = $st -> fetchrow_array ();
 	
@@ -89,17 +91,55 @@ sub sql_select_array {
 	my ($sql, @params) = @_;
 	my $st = $db -> prepare ($sql);
 	$st -> execute (@params);
-	my $result = $st -> fetchrow_array ();
+	my @result = $st -> fetchrow_array ();
 	$st -> finish;
 	
-	return $result;
+	return wantarray ? @result : $result [0];
 
 }
 
 ################################################################################
 
 sub sql_last_insert_id {
-	return sql_select_array ("SELECT LAST_INSERT_ID()");
+	return 0 + sql_select_array ("SELECT LAST_INSERT_ID()");
+}
+
+################################################################################
+
+sub sql_do_update {
+
+	my ($table_name, $field_list, $stay_fake) = @_;
+	my $sql = join ', ', map {"$_ = ?"} @$field_list;
+	$stay_fake or $sql .= ', fake = 0';
+	$sql = "UPDATE $table_name SET $sql WHERE id = ?";	
+	my @params = @_REQUEST {(map {"_$_"} @$field_list), 'id'};	
+	
+	sql_do ($sql, @params);
+	
+}
+
+################################################################################
+
+sub sql_do_insert {
+
+	my ($table_name, $pairs) = @_;
+		
+	my $fields = 'fake';
+	my $args   = '?';
+	my @params = $_REQUEST {sid};
+	
+	while (my ($field, $value) = each %$pairs) {
+	
+		$fields .= ', ' . $field;
+		$args   .= ', ?';
+		push @params, $value;
+	
+	}
+	
+	sql_do ("INSERT INTO $table_name ($fields) VALUES ($args)", @params);	
+	
+	return sql_last_insert_id ();
+	
 }
 
 1;
\ No newline at end of file
